import logging
import os
from pkg_resources import get_distribution, DistributionNotFound

from flask import Flask, current_app
from flask_restful import Api

from .manager import MountManager
from .resources import Mount,  SupportedLibraries
from .views import main

try:
    __version__ = get_distribution('thumbtack').version
except DistributionNotFound:
    __version__ = 'Could not find version'


def create_app(image_dir=None):
    app = Flask(__name__)
    app.config.from_object('thumbtack.config')
    # Since development doesn't have this environment variable, it won't do anything
    app.config.from_envvar('THUMBTACK_CONFIG_PRODUCTION', silent=True)

    IMAGE_DIR = os.environ.get("IMAGE_DIR")
    MOUNT_DIR = os.environ.get("MOUNT_DIR")

    if IMAGE_DIR:
        app.config.update(IMAGE_DIR=IMAGE_DIR)

    # image_dir is used in the thumbtack entry point, and overwrites the environment variable
    if image_dir:
        app.config.update(IMAGE_DIR=image_dir)

    if MOUNT_DIR:
        app.config.update(MOUNT_DIR=MOUNT_DIR)

    configure(app)

    app.before_first_request(before_first_request)

    return app


def configure(app):
    configure_extensions(app)
    configure_blueprints(app)
    app.logger.info('configured')


def configure_extensions(app):
    app.logger.info('configuring extensions')
    api = Api(app)

    mount_manager = MountManager()
    api.add_resource(Mount,
                     '/mounts/<path:image_path>',
                     '/mounts/',
                     resource_class_kwargs={'mount_manager': mount_manager})
    api.add_resource(SupportedLibraries, '/supported', endpoint='supported')


def configure_blueprints(app):
    app.logger.info('configuring blueprints')
    app.register_blueprint(main)


def before_first_request():
    configure_logging(current_app)


def configure_logging(app):
    if not app.debug:
        # In production mode, add log handler to sys.stderr.
        app.logger.setLevel(logging.DEBUG)

        formatter = logging.Formatter("[%(asctime)s] %(levelname)s in %(name)s.%(module)s: %(message)s")

        shandler = logging.StreamHandler()
        shandler.setLevel(logging.DEBUG)
        shandler.setFormatter(formatter)

        # app.logger.addHandler(fhandler)
        app.logger.addHandler(shandler)


def start_app():
    image_dir = os.getcwd()
    app = create_app(image_dir=image_dir)
    app.run(debug=True)



import os
import re

import requests

from flask import Blueprint, current_app, redirect, render_template, request, url_for

main = Blueprint('', __name__)

BASE_URL = 'http://localhost:5000'

STATUS_CODES = {
    200: 'Mounted',
    400: 'Unable to mount',
    404: 'Unmounted',
    500: 'Server error'
}


@main.route('/', methods=['GET'])
@main.route('/index', methods=['GET'])
def index():
    app = current_app._get_current_object()
    image_dir = app.config['IMAGE_DIR']
    supported_mount_types = []
    unsupported_mount_types = []

    url = '{}/supported'.format(BASE_URL)
    response = requests.get(url)
    if response.status_code == 200:
        json_response = response.json()
        for mount_type, supported in json_response.items():
            if supported:
                supported_mount_types.append(mount_type)
            else:
                unsupported_mount_types.append(mount_type)

    images = {}
    for root, dirs, files in os.walk(image_dir):
        for filename in files:

            rel_dir = os.path.relpath(root, image_dir)
            if rel_dir == '.':
                rel_file = os.path.join(rel_dir, filename)[2:]
            else:
                rel_file = os.path.join(rel_dir, filename)
            url = "{}/mounts/{}".format(BASE_URL, rel_file)

            full_path = os.path.join(root, filename)
            # Ignore *.E02, *.E03, ..., *.EAA, *.EAB, ..., but not *.E01
            if (not full_path.lower().endswith('log') and
                    re.match('.*\.[EL]X?\w\w$', full_path, flags=re.I) and
                    not re.match('.*\.[EL]X?01$', full_path, flags=re.I)):
                continue

            # Ignore *.002, *.003, ..., but not *.001
            if (re.match('.*\.\d\d\d$', full_path) and
                    not re.match('.*\.001$', full_path)):
                continue

            # Ignore *-sXXX.vmdk, but not *.vmdk
            if re.match('.*\-s\w\w\w\.vmdk$', full_path, flags=re.I):
                continue

            # Ignore  *-1.vhd, *-2.vhd, ...,  *-N.vhd, but not *-0.vhd
            if (re.match('.*\-\w+\.vhd$', full_path, flags=re.I) and
                    not re.match('.*\-0\.vhd$', full_path, flags=re.I)):
                continue

            if not filename.startswith('.') and os.path.isfile(full_path):
                response = requests.get(url)
                # response.raise_for_status()
                status = STATUS_CODES[response.status_code]

                json_response = response.json()
                # print(json_response)

                disk_mountpoint = ''
                ref_count = 0
                vol_mountpoints = {}
                if status == 'Mounted':
                    if 'mountpoint' in json_response['disk_info']:
                        disk_mountpoint = json_response['disk_info']['mountpoint']

                        for volume in json_response['disk_info']['volumes']:
                            sanitized_rel_file = rel_file.replace('/', '_').replace(':', '-').replace('.', '-')
                            uid = '{}_{}'.format(sanitized_rel_file, volume['index'])
                            vol_mountpoints[volume['index']] = [volume['mountpoint'], uid]

                    ref_count = json_response['ref_count']

                images[rel_file] = [full_path, status, disk_mountpoint, vol_mountpoints, ref_count]

    return render_template('index.html',
                           supported_mount_types=supported_mount_types,
                           unsupported_mount_types=unsupported_mount_types,
                           image_dir=image_dir,
                           images=images)


@main.route('/mount_form', methods=['POST'])
def mount_form():
    img_to_mount = request.form['img_to_mount']
    operation = request.form['operation']
    url = "{}/mounts/{}".format(BASE_URL, img_to_mount)
    status = None

    if operation == 'mount':
        response = requests.put(url)
        if response.status_code == 200:
            status = 'Mounted successfully'
    elif operation == 'unmount':
        response = requests.delete(url)
        if response.status_code == 200:
            status = 'Unmounted successfully'
    else:
        print('Unknown operation! How did you even get here!?')
        return redirect('/')

    if not status:
        status = 'Unable to complete operation'

    return render_template('form_complete.html',
                           status=status)




bind = "0.0.0.0:8081"
workers = 4
#env = None
#max-requests = None
#user = user
#group = group
logfile = "/var/log/thumbtack/thumbtack.log"
#loglevel = debug
IMAGE_DIR = '/vagrant/tests/test_images'
MOUNT_DIR = '/mnt/thumbtack'



import subprocess
import sys

from flask import current_app, request
from flask_restful import Resource, marshal_with, abort, fields

try:
    from pathlib import Path
except ImportError:
    from pathlib2 import Path

from .exceptions import MountManagerError


volume_fields = {
    'size': fields.Integer,
    'offset': fields.Integer,
    'index': fields.Integer,
    'label': fields.String(attribute=lambda obj: obj.info.get('label')),
    'fsdescription': fields.String(attribute=lambda obj: obj.info.get('fsdescription')),
    'fstype': fields.String,
    'mountpoint': fields.String,
}
disk_fields = {
    'name': fields.String(attribute='_name'),
    'imagepath': fields.String(attribute=lambda obj: obj.paths[0]),
    'mountpoint': fields.String,
    'volumes': fields.List(fields.Nested(volume_fields)),
}
disk_mount = {
    'disk_info': fields.Nested(disk_fields),
    'ref_count': fields.Integer
}


class Mount(Resource):
    """A Mount object that allows you to mount and unmount images.

    Attributes
    ----------
    mount_manager : thumbtack.MountManager
        The manager object tracks mounted state for all image
        files used with this Mount object.
    """

    def __init__(self, mount_manager):
        """Create a Mount object.

        Parameters
        ----------
        mount_manager : thumbtack.MountManager
            The manager object used that tracks mounted state
            of image files.
        """
        current_app.logger.info('Instantiating the Mount class')
        self.mount_manager = mount_manager

    @marshal_with(disk_fields)
    def put(self, image_path):
        """Mounts an image file.

        Parameters
        ----------
        image_path : str
            Path to an image file to be mounted. This is
            a relative path from the IMAGE_DIR config variable directory to the file on disk.
        mount_dir : str
            Path where the image file will be mounted.
        """
        try:
            mount_dir = request.args.get('mount_dir', '/mnt/thumbtack')
            print('mount_dir:', mount_dir)
            return self.mount_manager.mount_image(image_path, mount_dir)
        except MountManagerError as e:
            abort(400, message=str(e))

    @marshal_with(disk_mount)
    def get(self, image_path=None):
        """Retrieve information about tracked images.

        Parameters
        ----------
        image_path : str, optional
            Path to an image file.

        Returns
        -------
        dict
            Output from mount_manager.all_mounts() or
            mount_manager.get_mount(`image_path`)
        """
        if not image_path:
            response = []
            all_mounts = self.mount_manager.all_mounts()
            for mount in all_mounts:
                # have to include the trailing slash
                image_dir = '{}/'.format(current_app.config['IMAGE_DIR'])
                rel_path = mount.paths[0].replace(image_dir, '')
                ref_count = self.mount_manager.get_ref_count(rel_path)
                response.append({
                    'disk_info': mount,
                    'ref_count': ref_count
                })
            return response

        disk_info = self.mount_manager.get_mount(image_path)
        ref_count = self.mount_manager.get_ref_count(image_path)
        response = {
            'disk_info': disk_info,
            'ref_count': ref_count
        }
        return response

    def delete(self, image_path):
        """Unmounts an image file.

        Parameters
        ----------
        image_path : str
            Path to an image file to unmount. This is
            an absolute path to the file on disk.
        """
        self.mount_manager.unmount_image(image_path)


class SupportedLibraries(Resource):

    def get(self):

        virtualenv_bin_directory = Path(sys.argv[0]).parent
        imount_cmd = str(virtualenv_bin_directory / 'imount')

        # TODO: this seems... not right
        # gunicorn_powered = True if str(Path(sys.argv[0]).name) == 'gunicorn' else False
        # if gunicorn_powered:
        #     current_app.logger.info('powered by gunicorn')
        #     imount_cmd = str(Path(sys.argv[0]).parent / 'imount')

        run_args = [imount_cmd, '--check']

        current_app.logger.info(run_args)
        output = subprocess.check_output(run_args).decode().split('\n')
        outputjson = {}

        for line in output:
            lineSplit = line.split()
            if len(lineSplit) > 0 and ('MISSING' in lineSplit or 'INSTALLED' in lineSplit):
                if lineSplit[0] == 'INSTALLED':
                    outputjson[str(lineSplit[1])] = True
                else:
                    outputjson[str(lineSplit[1])] = False

        return outputjson



from collections import namedtuple

import imagemounter
import os
from flask import current_app
from flask_restful import abort

from .exceptions import MountManagerError


class MountManager(object):
    """Built from an `ImageParser <http://imagemounter.readthedocs.io/en/latest/python.html#imageparser>`_ object, the
    MountManager object tracks the mounted state for all image files used with the object.

    Attributes
    ----------
    ImageParser : imagemounter.ImageParser
        The root object of the `imagemounter <http://imagemounter.readthedocs.io/en/latest/python.html#imageparser>`_ Python interface.
        The ImageParser object has access to all functions in the imagemounter.parser.py
        module.
    """
    MountInfo = namedtuple('MountInfo', ('parser', 'ref_count'))

    def __init__(self):
        self.mounts = {}

    def mount_image(self, relative_image_path, mount_dir='/mnt/thumbtack'):
        """
           First checks if the image is already mounted,
           then verifies that the file path is valid.

        Parameters
        ----------
        relative_image_path : str
            Path to an image file. This is a relative path to the file on disk from the IMAGE_DIR.
        mount_dir : str
            Path where the image file will be mounted.

        Returns
        -------
        An imagemounter.ImageParser.Disk object
            the `imagemounter.Disk <http://imagemounter.readthedocs.io/en/latest/python.html#disk>`_ just mounted

        """
        full_image_path = '{}/{}'.format(current_app.config['IMAGE_DIR'], relative_image_path)

        # See if this image is already mounted
        mount_info = self.mounts.get(relative_image_path)
        if mount_info is not None:
            new_ref_count = mount_info.ref_count + 1
            current_app.logger.info(
                'MountManager.mount_image: image_path "{}" is already mounted. Increasing ref count to {:d}.'.format(
                    relative_image_path, new_ref_count))
            self.mounts[relative_image_path] = self.MountInfo(parser=mount_info.parser, ref_count=new_ref_count)
            return mount_info.parser.disks[0]

        # Verify we have a valid file path
        if not os.access(full_image_path, os.R_OK):
            msg = 'MountManager.mount_image: "{}" is not a valid file or is not accessible for reading'.format(
                relative_image_path)
            current_app.logger.info(msg)
            raise MountManagerError(msg)

        # Mount it
        current_app.logger.info('MountManager.mount_image: mounting image_path "{}"'.format(relative_image_path))
        # Updates: full_image_path needs to be iterable, pretty keeps path names the way they were
        image_parser = imagemounter.ImageParser([full_image_path], pretty=True, mountdir=mount_dir)

        # Volumes won't be mounted unless this generator is iterated
        for _ in image_parser.init():
            pass

        if len(image_parser.disks) != 1:
            current_app.logger.info('MountManager.mount_image: Error: Unexpected number of disks (expected 1, got {:d})'.format(
                len(image_parser.disks)))
            image_parser.clean(allow_lazy=True)
            raise MountManagerError('Unexpected number of disks (expected 1, got {:d})'.format(len(image_parser.disks)))

        current_app.logger.info("MountManager.mount_image: Disk Mounted: {}".format(image_parser.disks[0].mountpoint))
        for volume in image_parser.disks[0].volumes:
            current_app.logger.info("MountManager.mount_image: Volume description: {}".format(volume.get_description()))
            current_app.logger.info("MountManager.mount_image: Volume mountpoint: {}".format(
                volume.mountpoint if volume.mountpoint else '<none>'))

        # Fail if we couldn't mount any of the volumes
        if not [v for v in image_parser.disks[0].volumes if v.mountpoint]:
            image_parser.clean(allow_lazy=True)
            current_app.logger.error('MountManager.mount_image: No mountable volumes in image "{}"'.format(relative_image_path))
            raise MountManagerError('Disk image contains no mountable volumes')

        self.mounts[relative_image_path] = self.MountInfo(parser=image_parser, ref_count=1)
        return image_parser.disks[0]

    def get_mount(self, relative_image_path):
        """Retrieves the disk object at the given image path

            Parameters
            ----------
            image_path : str
                Path to the image file. This is
                an absolute path to the file on disk.
            Returns
            -------
            imagemounter.ImageParser.disk object
                the `imagemounter.Disk <http://imagemounter.readthedocs.io/en/latest/python.html#disk>`_ at the specified image path
        """
        mount_info = self.mounts.get(relative_image_path)
        if mount_info is not None:
            return mount_info.parser.disks[0]
        else:
            abort(404, message='Image path {} is not mounted'.format(relative_image_path))

    def unmount_image(self, relative_image_path):
        """Unmounts by using the supplied image path as an index into the collection of mounted images, and then removes itself from the collection.

        Parameters
        ----------
        image_path : str
            Path to the image file. This is
            an absolute path to the file on disk.

        """
        mount_info = self.mounts.get(relative_image_path)
        if mount_info is None:
            abort(404, message='Image path {} is not mounted'.format(relative_image_path))

        if mount_info.ref_count > 1:
            new_ref_count = mount_info.ref_count - 1
            current_app.logger.info(
                'MountManager.unmount_image: image_path "{}" is still referenced. Decreasing ref count to {:d}.'.format(
                    relative_image_path, new_ref_count))
            self.mounts[relative_image_path] = self.MountInfo(parser=mount_info.parser, ref_count=new_ref_count)
        else:
            current_app.logger.info('MountManager.unmount_image: unmounting image_path "{}"'.format(relative_image_path))
            mount_info.parser.clean(allow_lazy=True)
            del self.mounts[relative_image_path]

    def all_mounts(self):
        """Retrieves disk object for each disk with a valid image path

        Returns
        -------
        imagemounter.ImageParser.Disk objects
            collection of valid `Disk <http://imagemounter.readthedocs.io/en/latest/python.html#disk>`_ objects
            associated with this MountManager

        """
        return [mi.parser.disks[0] for mi in self.mounts.values()]

    def get_ref_count(self, relative_image_path):
        mount_info = self.mounts.get(relative_image_path)
        if mount_info is not None:
            return mount_info.ref_count
        return 0

    def cleanup(self):
        """Uses `imagemounter.ImageParser.clean <http://imagemounter.readthedocs.io/en/latest/python.html#imageparser>`_ to
        unmount Disk objects associated with this MountManger

        """
        current_app.logger.info('MountManager.cleanup: cleaning remaining mounts')
        for image_path, mount_info in self.mounts.iteritems():
            current_app.logger.warning(
                'MountManager.cleanup: image_path "{}" is being unmounted with {:d} outstanding references.'.format(
                    image_path, mount_info.ref_count))
            mount_info.parser.clean(allow_lazy=True)



import json
import os

from pprint import pprint

import pytest


def test_find_library_support(test_client):
    """
    GIVEN a Thumbtack Flask application client
    WHEN the '/supported' page is requested (GET)
    THEN check that the page shows which libraries it can check
    """
    # NOTE: this test does not verify support for individual libraries,
    # only that the page itself exists
    print('################  Test Find Library Support')
    response = test_client.get('/supported')
    print('response', type(response), response)
    assert response.status_code == 200

    potential_libraries = ['xmount', 'ewfmount', 'affuse', 'vmware-mount', 'mountavfs', 'qemu-nbd', 'mmls', 'pytsk3',
                           'parted', 'fsstat', 'file', 'blkid', 'python-magic', 'disktype', 'mount.xfs', 'mount.ntfs',
                           'lvm', 'vmfs-fuse', 'mount.jffs2', 'mount.squashfs', 'mdadm', 'cryptsetup', 'bdemount',
                           'vshadowmount']
    data = json.loads(response.data.decode('utf-8'))
    for lib in potential_libraries:
        assert lib in data


def test_get_info_empty_mount_list(test_client):
    """
    GIVEN a Thumbtack Flask application client
    WHEN the '/mounts' page is requested (GET)
    THEN check the response code is valid and no images are mounted
    """
    print('################  Test Get Info Empty Mount List')
    response = test_client.get('/mounts/')
    assert response.status_code == 200
    assert json.loads(response.data.decode('utf-8')) == []


def test_get_info_nonexistent_image(test_client):
    """
    GIVEN a Thumbtack Flask application client
    WHEN information is requested about a non-existent disk image (GET)
    THEN check the response code is valid and no images are mounted
    """
    print('################  Test Get Invalid Image')
    response = test_client.get('/mounts/nonexistent/disk/image.E01')
    assert response.status_code == 404


def test_mount_nonexistent_image(test_client):
    """
    GIVEN a Thumbtack Flask application client
    WHEN a non-existent disk image is requested to be mounted (PUT)
    THEN check the response code is valid and no images are mounted
    """
    print('\n################  Test Mount Non-existent Image')
    response = test_client.put('/mounts/nonexistent/disk/image.E01')
    assert response.status_code == 400

    response = test_client.get('/mounts/')
    assert response.status_code == 200
    assert json.loads(response.data.decode('utf-8')) == []


def test_mount_invalid_image(test_client):
    """
    GIVEN a Thumbtack Flask application client
    WHEN an invalid disk image (text file) is requested to be mounted (PUT)
    THEN check the response code is valid and no images are mounted
    """
    print('\n################  Test Mount Invalid Image')
    response = test_client.put('/mounts/foo.dd')
    assert response.status_code == 400

    response = test_client.get('/mounts/')
    assert response.status_code == 200
    assert json.loads(response.data.decode('utf-8')) == []


@pytest.mark.parametrize("test_image_path", [
    'dftt_images/10-ntfs-disk.dd',
    'dftt_images/11-carve-fat.dd',
    'dftt_images/12-carve-ext2.dd',
    'dftt_images/7-ntfs-undel.dd',
    'dftt_images/8-jpeg-search.dd',
    'dftt_images/9-fat-label.dd',
    'dftt_images/daylight.dd',
    'dftt_images/ext-part-test-2.dd',
    'dftt_images/ntfs-img-kw-1.dd',
    'digitalcorpora/lone_wolf/LoneWolf.E01',
    'digitalcorpora/m57-jean/nps-2008-jean.E01',
    'digitalcorpora/m57-patents/charlie-2009-12-11.E01',
    'digitalcorpora/m57-patents/charlie-work-usb-2009-12-11.E01',
    'digitalcorpora/m57-patents/jo-2009-12-11-001.E01',
    'digitalcorpora/m57-patents/jo-favorites-usb-2009-12-11.E01',
    'digitalcorpora/m57-patents/pat-2009-12-11.E01',
    'digitalcorpora/m57-patents/terry-2009-12-11-001.E01',
    'digitalcorpora/m57-patents/terry-work-usb-2009-12-11.E01',
    'digitalcorpora/national_gallery/tracy-home-2012-07-16-final.E01'
])
def test_mount_valid_images(test_client, expected_test_results, test_image_path):
    """
    GIVEN a Thumbtack Flask application client
    WHEN a disk image is requested to be mounted (PUT)
    THEN check the response code is valid
    """

    print('\n################  Test Mount Valid Images')

    if not os.path.isfile(os.path.join('test_images', test_image_path)):
        pytest.skip('Skipping disk mounting service test because test disk image is missing ({})'.format(test_image_path))

    for disk_image_info in expected_test_results:
        if disk_image_info['expected_json_response']['imagepath'] == test_image_path:
            expected_json_results = disk_image_info['expected_json_response']
            assertions = disk_image_info['assertions']

            # ignore the invalid images from the expected json results file
            if not assertions['mountable']:
                continue

            relative_image_path = expected_json_results['imagepath']
            # update imagepath in expected results to be full path because that is what is returned
            expected_json_results['imagepath'] = '{}/{}'.format(test_client.application.config['IMAGE_DIR'],
                                                                expected_json_results['imagepath'])

            print('\n############################  Mounting: {}'.format(relative_image_path))

            # MOUNT IMAGE
            # Verify PUT /mounts/<TEST_DISK_IMAGE> returns the new mount
            mount_dir = test_client.application.config['MOUNT_DIR']
            print('mounting: {}'.format(relative_image_path))
            # response = test_client.put(u'/mounts/{}'.format(relative_image_path))
            response = test_client.put(u'/mounts/{}?mount_dir={}'.format(relative_image_path, mount_dir))
            assert response.status_code == 200

            response_json = json.loads(response.data.decode('utf-8'))
            disk_mountpoint = response_json.get('mountpoint')

            volumes = response_json['volumes']
            for volume in volumes:
                volume.pop('mountpoint')  # removing volume mountpoints because values are randomized

            # volume_mountpoint = volumes[0].pop('mountpoint')  # removing mountpoint because value is randomized

            num_volumes = len(volumes)
            del response_json['mountpoint']  # removing disk mountpoint because value is randomized

            print('disk_mountpoint: {}'.format(disk_mountpoint))
            print('volumes: {}'.format(num_volumes))

            assert disk_mountpoint is not None
            assert num_volumes == assertions['num_volumes']
            assert response_json == expected_json_results
            for volume in volumes:
                if not volume.get('mountpoint') in [None, '']:
                    assert os.access(volume['mountpoint'], os.R_OK)  # Verify the mount point can be accessed

            # GET MOUNTED IMAGE INFO
            # Verify GET /mounts/ returns the new mount
            response = test_client.get(u'/mounts/{}'.format(relative_image_path))
            assert response.status_code == 200

            response_json = json.loads(response.data.decode('utf-8'))
            # pprint(response_json)
            disk_info = response_json['disk_info']
            del disk_info['mountpoint']
            assert len(disk_info['volumes']) == assertions['num_volumes']

            volumes = disk_info['volumes']
            all_volume_mountpoints = [x['mountpoint'] for x in volumes if x['mountpoint'] != '']
            for volume in volumes:
                del volume['mountpoint']  # removing because value is randomized
            assert disk_info == expected_json_results

            # UNMOUNT IMAGE
            # Verify DELETE /mounts/<TEST_DISK_IMAGE> unmounts the image
            response = test_client.delete(u'/mounts/{}'.format(relative_image_path))
            assert response.status_code == 200

            # Verify the mount point is gone
            for volume_mountpoint in all_volume_mountpoints:
                assert not os.access(volume_mountpoint, os.R_OK)


def test_mount_images_with_unmountable_volumes(test_client, expected_test_results):
    """
    GIVEN a Thumbtack Flask application client
    WHEN a valid disk image that only has volumes that are unmountable are requested to be mounted (PUT)
    THEN check the response code is 400
    """

    print('\n################  Test Mount Valid Images with unmountable volumes')

    for disk_image_info in expected_test_results:

        expected_json_results = disk_image_info['expected_json_response']
        assertions = disk_image_info['assertions']

        # ignore the valid images from the expected json results file
        if assertions['mountable']:
            continue

        relative_image_path = expected_json_results['imagepath']
        # update imagepath in expected results to be full path because that is what is returned
        expected_json_results['imagepath'] = '{}/{}'.format(test_client.application.config['IMAGE_DIR'], expected_json_results['imagepath'])

        print('\n############################  Mounting: {}'.format(relative_image_path))

        if not os.path.isfile(os.path.join('test_images', relative_image_path)):
            # TODO: this will skip the whole test if any one file is missing. Needs to be addressed
            print('skipping: {}'.format(relative_image_path))
            pytest.skip('Skipping disk mounting service test because test disk image is missing ({})'.format(relative_image_path))

        # MOUNT IMAGE
        # Verify PUT /mounts/<TEST_DISK_IMAGE> returns the new mount
        print('mounting: {}'.format(relative_image_path))
        response = test_client.put(u'/mounts/{}'.format(relative_image_path))
        assert response.status_code == 400



import io
import os

from setuptools import setup, find_packages

# Package meta-data.
NAME = 'thumbtack'
DESCRIPTION = 'Service to manage disk image mounts.'
URL = 'https://github.com/mitre/thumbtack'
EMAIL = 'thumbtack@mitre.org'
AUTHOR = 'The MITRE Corporation'
LICENSE = 'Apache 2.0'
REQUIRES_PYTHON = '>=2.7.0'
VERSION = '0.1.4'

REQUIRED = [
    'Flask',
    'Flask-RESTful',
    'gunicorn',
    'imagemounter',
    'requests',
    'pathlib2 ; python_version<"3.4"',
]

doc_requires = [
    'sphinx',
]

test_requires = [
    'coverage',
    'pytest',
    'pytest-cov',
]

dev_requires = doc_requires + test_requires + [
    'bumpversion',
    'python-magic',
    'pytsk3',
]

EXTRAS = {
    'dev': dev_requires,
    'docs': doc_requires,
    'test': test_requires,
}

here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
try:
    with io.open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION


setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    license=LICENSE,
    long_description=long_description,
    long_description_content_type='text/x-rst',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Security',
        'Topic :: System :: Filesystems',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7'
    ],
    packages=find_packages(exclude=('tests',)),
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'thumbtack = thumbtack:start_app',
        ]
    }
)
